// MainFrm.h : interface of the CMainFrame class
//
/////////////////////////////////////////////////////////////////////////////

#pragma once

class CMainFrame : public CFrameWindowImpl<CMainFrame>, public CUpdateUI<CMainFrame>,
		public CMessageFilter, public CIdleHandler
{
	CMenu menu, device_menu, view_menu;
	static const int view_menu_wid = 99;
	bool always_on_top;

public:
	DECLARE_FRAME_WND_CLASS(NULL, IDR_MAINFRAME)

	CFctrl_wtlView m_view;

	virtual BOOL PreTranslateMessage(MSG* pMsg)
	{
		if(CFrameWindowImpl<CMainFrame>::PreTranslateMessage(pMsg))
			return TRUE;

		return m_view.PreTranslateMessage(pMsg);
	}

	virtual BOOL OnIdle()
	{
		return FALSE;
	}

	BEGIN_UPDATE_UI_MAP(CMainFrame)
	END_UPDATE_UI_MAP()

	BEGIN_MSG_MAP(CMainFrame)
		MESSAGE_HANDLER(WM_CREATE, OnCreate)
		MESSAGE_HANDLER(WM_DESTROY, OnDestroy)
		MESSAGE_HANDLER(WM_COMMAND, OnCommand)
		MESSAGE_HANDLER(WM_NCLBUTTONDBLCLK, OnNcLButtonDblClk)
		CHAIN_MSG_MAP(CUpdateUI<CMainFrame>)
		CHAIN_MSG_MAP(CFrameWindowImpl<CMainFrame>)
		CHAIN_CLIENT_COMMANDS()
	END_MSG_MAP()

// Handler prototypes (uncomment arguments if needed):
//	LRESULT MessageHandler(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
//	LRESULT CommandHandler(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
//	LRESULT NotifyHandler(int /*idCtrl*/, LPNMHDR /*pnmh*/, BOOL& /*bHandled*/)

	LRESULT OnCreate(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
	{
		// Create View
		CRect rect;
		rect.SetRect(0, 0, 430, 240);
		m_hWndClient = m_view.Create(m_hWnd, rect, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN, WS_EX_CLIENTEDGE);
		this->ResizeClient(rect.Width(), rect.Height(), false);

		// register object for message filtering and idle updates
		CMessageLoop* pLoop = _Module.GetMessageLoop();
		ATLASSERT(pLoop != NULL);
		pLoop->AddMessageFilter(this);
		pLoop->AddIdleHandler(this);

		// Create Menu
		CMenuItemInfo miinfo;

		menu.CreateMenu();
		device_menu.CreateMenu();
		miinfo.fMask = MIIM_TYPE | MIIM_SUBMENU;
		miinfo.fType = MFT_STRING;
		miinfo.hSubMenu = device_menu;
		miinfo.dwTypeData = _T("&Device");
		menu.InsertMenuItemW(0, true, &miinfo);
		
		MIDIOUTCAPS midi_out_caps;
		miinfo.fMask = MIIM_TYPE | MIIM_STATE | MIIM_ID;
		for (UINT i=0; i<midiOutGetNumDevs(); i++) {
			midiOutGetDevCaps(i, (LPMIDIOUTCAPS)&midi_out_caps, sizeof(midi_out_caps));
			miinfo.dwTypeData = midi_out_caps.szPname;
			miinfo.wID = i;
			if (i == m_view.GetMidiDeviceId()) {
				miinfo.fState = MFS_CHECKED;
			} else {
				miinfo.fState = MFS_UNCHECKED;
			}
			device_menu.InsertMenuItem(i, true, &miinfo);
		}
		
		view_menu = CreateMenu();
		miinfo.fMask = MIIM_TYPE | MIIM_SUBMENU;
		miinfo.hSubMenu = view_menu;
		miinfo.dwTypeData = _T("&View");
		InsertMenuItem(menu, 1, true, &miinfo);		

		miinfo.fMask = MIIM_TYPE | MIIM_STATE | MIIM_ID;
		miinfo.dwTypeData = _T("Always On Top");
		miinfo.fState = MFS_CHECKED;
		miinfo.wID = view_menu_wid;
		InsertMenuItem(view_menu, 0, true, &miinfo);

		SetMenu(menu);

		// Always on Top
		always_on_top = true;
		SetWindowPos(HWND_TOPMOST,0,0,0,0,SWP_NOMOVE | SWP_NOSIZE);

		return 0;
	}

	LRESULT OnCommand(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/)
	{
		CMenuItemInfo miinfo;
		miinfo.fMask = MIIM_STATE;
		if (LOWORD(wParam) == view_menu_wid) {
			// View(Always On Top)
			if (always_on_top) {
				always_on_top = false;
				miinfo.fState = MFS_UNCHECKED;
				SetWindowPos(HWND_NOTOPMOST,0,0,0,0,SWP_NOMOVE | SWP_NOSIZE);
			} else {
				always_on_top = true;
				miinfo.fState = MFS_CHECKED;
				SetWindowPos(HWND_TOPMOST,0,0,0,0,SWP_NOMOVE | SWP_NOSIZE);
			}
			SetMenuItemInfo(view_menu, 0, true, &miinfo);
		} else {
			// Device
			m_view.SetMidiDeviceId(LOWORD(wParam));
			midiOutReset((*m_view.GetMidiOut()));
			midiOutClose((*m_view.GetMidiOut()));
			midiOutOpen(m_view.GetMidiOut(), m_view.GetMidiDeviceId(), 0, 0, 0);
			for (UINT i=0; i < midiOutGetNumDevs(); i++) {
				if (i == m_view.GetMidiDeviceId() ) {
					miinfo.fState = MFS_CHECKED;
				} else {
					miinfo.fState = MFS_UNCHECKED;
				}
				SetMenuItemInfo(device_menu,i,true,&miinfo);
			}
		}
		return 0;
	}

	LRESULT OnDestroy(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled)
	{
		// unregister message filtering and idle updates
		CMessageLoop* pLoop = _Module.GetMessageLoop();
		ATLASSERT(pLoop != NULL);
		pLoop->RemoveMessageFilter(this);
		pLoop->RemoveIdleHandler(this);

		bHandled = FALSE;
		return 1;
	}

	// Window Shade
	LRESULT OnNcLButtonDblClk(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/)
	{
		CRect rect;
		static int pheight;
		if (wParam == HTCAPTION ) {
			GetClientRect(rect);
			if (rect.bottom != 0) {
				GetWindowRect(rect);
				pheight = rect.bottom - rect.top;
				MoveWindow(rect.left,
							rect.top,
							rect.right - rect.left,
							0, TRUE);
			} else {
				GetWindowRect(rect);
				MoveWindow(rect.left,
							rect.top,
							rect.right - rect.left,
							pheight, TRUE);
			}
		}
		return 0;
	}

};
