// Virtual Base Object
class Object
{	
protected:
	int		number;
	CRect	rect;
	bool	selected;
	bool	moved;

public:
	Object(int& Number, CRect& /*client_rect*/) {
		number = Number;
		selected = false;
	}

	virtual ~Object() {};
	virtual void Paint(CDCHandle& dc) {};
	virtual void SetPointer(CPoint& point, Object*& p_object, int& corner_num) {};
	virtual void Move(CSize size, Object*& p_object, int& corner_num, CRect& client_rect) {};
	virtual void InSelectionRect(CRect& selection_rect) {};
	
	int GetNumber() { return number; }
	void SetSelected(bool bl) { selected = bl; }
	bool GetSelected() { return selected; }
	bool GetMoved() { return moved; }
	CRect GetRect() { return rect; }
};


// Channel Object
class ObjectCH : public virtual Object
{
	CRect	inner_rect, corner_rect[4];

public:
	ObjectCH::ObjectCH(int& number, CRect& client_rect) : Object(number, client_rect) {

		// Set Outer Rect
		CPoint point;
		point.SetPoint(client_rect.Width()/2 + (client_rect.Width() * number), 
						client_rect.Height()/2 + (client_rect.Height() * number));
		rect.SetRect(point.x - client_rect.Width()/4,
					 point.y - client_rect.Height()/4,
					 point.x + client_rect.Width()/4,
					 point.y + client_rect.Height()/4);

		// Set Inner Rect
		inner_rect.CopyRect(rect);
		inner_rect.DeflateRect(rect.Width()/4,rect.Height()/4);
		
		// Set Corner Rect
		//  0----3
		//  |    |
		//  1----2
		const int corner_size = 10;
		corner_rect[0].SetRect(rect.left, rect.top,
							   rect.left + corner_size, rect.top + corner_size);
		corner_rect[1].SetRect(rect.left, rect.bottom - corner_size,
							   rect.left + corner_size, rect.bottom);
		corner_rect[2].SetRect(rect.right - corner_size, rect.bottom - corner_size,
							   rect.right, rect.bottom);
		corner_rect[3].SetRect(rect.right - corner_size, rect.top,
							   rect.right, rect.top + corner_size);
	}

	void Paint(CDCHandle& dc) {
		// Paint Rect
		CBrush brush;
		brush.CreateSolidBrush(RGB(80,80,80));
		HBRUSH hOldBrush = dc.SelectBrush(brush);
	
		for (int i=0; i<4; i++)						// Corner Rect
			FrameRect(dc,corner_rect[i],brush);	
		
		FrameRect(dc,inner_rect,brush);				// Inner Rect

		brush.DeleteObject();						// Outer Rect
		if (selected == false) {
			brush.CreateSolidBrush(RGB(100,100,100));
		} else {
			brush.CreateSolidBrush(RGB(200,200,200));
		}
		FrameRect(dc,rect,brush);
		dc.SelectBrush(hOldBrush);
		brush.DeleteObject();

		// Paint Font
        CFont font;
		CString string;
        font.CreatePointFont(80, _T("Arial"),0,false,true);
        CFont hOldFont = dc.SelectFont(font);		
		dc.SetTextColor(RGB(120, 120, 120));
		string.Format(_T("ch.%d"), number + 1);
		dc.TextOutW(rect.right - 40,rect.bottom - 16, string, string.GetLength());
		dc.SelectFont(hOldFont);
		font.DeleteObject();
	}

	void SetPointer(CPoint& point, Object*& p_object, int& corner_num) {
		// Outer Rect
		if (rect.PtInRect(point)) {
			p_object = this;
			// Corner Rect
			for (int i=0; i<4; i++) {
				if (corner_rect[i].PtInRect(point)) {
					corner_num = i;
					break;
				}
			}
		}
	}

	void Move(CSize size, Object*& p_object, int& corner_num, CRect& client_rect) {
		moved = false;
		if (selected) {
			if ((p_object == this) && (corner_num != -1)) {
				// Corner Move
				switch (corner_num) {
				case 0:
					size.cx = -size.cx;
					size.cy = -size.cy;
					break;
				case 1:
					size.cx = -size.cx;
					break;
				case 2:
					break;
				case 3:
					size.cy = -size.cy;
					break;
				}

				const int limit = 64;
				if (rect.Width() + size.cx * 2 < limit)
					size.cx = (rect.Width() - limit) / 2;
				if (rect.Height() + size.cy * 2 < limit / 2)
					size.cy = (rect.Height() - limit / 2) / 2;

				if ((size.cx != 0) || (size.cy != 0)) {
					rect.InflateRect(size);
					inner_rect.CopyRect(rect);
					inner_rect.DeflateRect(rect.Width()/4,rect.Height()/4);
					corner_rect[0].OffsetRect(-size.cx, -size.cy);
					corner_rect[1].OffsetRect(-size.cx, size.cy);
					corner_rect[2].OffsetRect(size.cx, size.cy);
					corner_rect[3].OffsetRect(size.cx, -size.cy);
				
					moved = true;
				}

			} else {
				// Offset Move
				const int intLimit = 16;
				if ((client_rect.right - rect.left - size.cx) <= intLimit) {
					size.cx = client_rect.right - rect.left - intLimit;
				} else if ((client_rect.left - rect.right - size.cx) >= -intLimit) {
					size.cx = client_rect.left - rect.right + intLimit;
				}
				if ((client_rect.bottom - rect.top - size.cy) <= intLimit) {
					size.cy = client_rect.bottom - rect.top - intLimit;
				} else if ((client_rect.top - rect.bottom - size.cy) >= -intLimit) {
					size.cy = client_rect.top - rect.bottom + intLimit;
				}
				
				if ((size.cx != 0) || (size.cy != 0)) {
					rect.OffsetRect(size);
					inner_rect.OffsetRect(size);
					for (int i=0; i<4; i++)
						corner_rect[i].OffsetRect(size);
			
					moved = true;
				}
			}
		}
	}

	void InSelectionRect(CRect& selection_rect) {
		if (selection_rect.PtInRect(rect.TopLeft()) && 
			selection_rect.PtInRect(rect.BottomRight()))
			selected = true;
	}
};


// Control Change Object
class ObjectCC : public virtual Object
{
public:
	ObjectCC::ObjectCC(int& number, CRect& client_rect) : Object(number, client_rect) {
		//Set Default Rect
		int cc_width = 16;
		rect.SetRect(30 * number + 20,
					 10,
					 30 * number + 20 + cc_width,
					 10 + cc_width);
	}

	void Paint(CDCHandle& dc) {	
		// Paint Font
        CFont font;
		CString string;
        font.CreatePointFont(90, _T("Arial"),0,false,false);
        CFont old_font = dc.SelectFont(font);
		if (selected == false) {
			dc.SetTextColor(RGB(120, 120, 120));
		} else {
			dc.SetTextColor(RGB(250, 250, 250));
		}
		string.Format(_T("%d"), number);
		SetTextAlign(dc, TA_CENTER);
		dc.TextOutW(rect.CenterPoint().x, rect.top + 1, string, string.GetLength());
		dc.SelectFont(old_font);
		font.DeleteObject();
	}

	void SetPointer(CPoint& point, Object*& p_object, int& corner_num) {
		if (rect.PtInRect(point)) {
			p_object = this;
			corner_num = -1;
		}
	}

	void Move(CSize size, Object*& /*p_object*/, int& corner_num, CRect& client_rect) {
		moved = false;
		if (selected && (corner_num == -1)) {
			if ((rect.CenterPoint().x + size.cx) >= client_rect.right) {
				size.cx = client_rect.right - rect.CenterPoint().x;
			} else if ((rect.CenterPoint().x + size.cx) <= client_rect.left) {
				size.cx = client_rect.left - rect.CenterPoint().x;
			}
			if (client_rect.bottom <= (rect.CenterPoint().y + size.cy)) {
				size.cy = client_rect.bottom - rect.CenterPoint().y;
			} else if (client_rect.top >= (rect.CenterPoint().y + size.cy)) {
				size.cy = client_rect.top - rect.CenterPoint().y;
			}

			if ((size.cx != 0) || (size.cy != 0)) {
				rect.OffsetRect(size);

				moved = true;
			}
		}
	}

	void InSelectionRect(CRect& selection_rect) {
		if (selection_rect.PtInRect(rect.CenterPoint()))
			selected = true;
	}
};

