// f.ctrl_wtlView.h : interface of the CFctrl_wtlView class
//
/////////////////////////////////////////////////////////////////////////////

#pragma once

#define MIDIMSG(stat , data1 , data2) (DWORD)(stat | (data1 << 8) | (data2 << 16))

class CFctrl_wtlView : public CDoubleBufferWindowImpl<CFctrl_wtlView>
{
	int midi_device_id;
	HMIDIOUT midi_out;

	bool mouse_click;
	CPoint prev_point;
	Object *p_object;	// Pointer to Object under cursor.
	int	corner_num;		// Corner number under cursor.
	CRect selection_rect;

	// Layer
	std::vector<std::vector<Object *> *> vecLayer;
	std::vector<std::vector<Object *> *>::iterator itLayer;

	// Object
	static const int COUNT_CH = 2;
	static const int COUNT_CC = 8;

	std::vector<Object *> vecCH;
	std::vector<Object *> vecCC;
	std::vector<Object *>::iterator itObject;

public:
	inline void SetMidiDeviceId(int id) { midi_device_id = id; }
	inline int GetMidiDeviceId() { return midi_device_id; }
	inline HMIDIOUT *GetMidiOut() { return &midi_out; }

	DECLARE_WND_CLASS(NULL)

	BOOL PreTranslateMessage(MSG* pMsg)
	{
		pMsg;
		return FALSE;
	}

	BEGIN_MSG_MAP_EX(CFctrl_wtlView)
		MSG_WM_CREATE(OnCreate)
		MSG_WM_DESTROY(OnDestroy)
		MSG_WM_LBUTTONDOWN(OnLButtonDown)
		MSG_WM_LBUTTONUP(OnLButtonUp)
		MSG_WM_SETCURSOR(OnSetCursor)
		MSG_WM_MOUSEMOVE(OnMouseMove)

		CHAIN_MSG_MAP(CDoubleBufferWindowImpl<CFctrl_wtlView>)
	END_MSG_MAP()


    int OnCreate(LPCREATESTRUCT lpCreateStruct)
	{
        LRESULT lRet = DefWindowProc();

		midi_device_id = 0;
		midiOutOpen(&midi_out, midi_device_id, 0, 0, 0);

		mouse_click = false;

		CRect client_rect;
		GetClientRect(client_rect);

		// Create CH
		for (int i=0; i<COUNT_CH; i++) {
			vecCH.push_back( new ObjectCH(i, client_rect));
		}

		// Create CC
		for (int i=0; i<COUNT_CC; i++) {
			vecCC.push_back( new ObjectCC(i, client_rect));
		}

		// Push objects to layer
		vecLayer.push_back (&vecCH);
		vecLayer.push_back (&vecCC);

        return lRet;
    }


	void OnDestroy()
	{
		midiOutReset(midi_out);
		midiOutClose(midi_out);

		// Delete Objects
		itLayer = vecLayer.begin();
		while (itLayer != vecLayer.end())
		{
			itObject = (*itLayer)->begin();
			while (itObject != (*itLayer)->end())
			{
				delete *itObject;
				itObject++;
			}
			itLayer++;
		}
	}


	void OnLButtonDown(UINT nFlags, CPoint point)
	{
		mouse_click = true;
		prev_point = point;
		SetCapture();

		if (p_object != NULL) {
			// Select
			itLayer = vecLayer.begin();
			while (itLayer != vecLayer.end())
			{
				itObject = (*itLayer)->begin();
				while (itObject != (*itLayer)->end())
				{
							// Ctrl Down
					if ((GetKeyState(VK_CONTROL) >= 0) && (p_object->GetSelected() == false)) {
						(*itObject)->SetSelected(false);
					}
					itObject++;
				}
				itLayer++;
			}		

					// Ctrl Not Down
			if ((GetKeyState(VK_CONTROL) < 0) && (p_object->GetSelected())) {
				p_object->SetSelected(false);
			} else {
				p_object->SetSelected(true);
			}

		} else {
			// Unselect All
			itLayer = vecLayer.begin();
			while (itLayer != vecLayer.end())
			{
				itObject = (*itLayer)->begin();
				while (itObject != (*itLayer)->end())
				{
					(*itObject)->SetSelected(false);
					itObject++;
				}
				itLayer++;
			}
		}

		CRect client_rect;
		GetClientRect(client_rect);
		InvalidateRect(client_rect, false);
	}


	void OnLButtonUp(UINT nFlags, CPoint point)
	{
		mouse_click = false;
		ReleaseCapture();
		selection_rect.SetRectEmpty();
		CRect client_rect;
		GetClientRect(client_rect);
		InvalidateRect(client_rect,false);
	}


	BOOL OnSetCursor(CWindow wnd, UINT nHitTest, UINT message)
	{
		switch (corner_num) {
		case -1:
			SetCursor(LoadCursor(NULL,IDC_ARROW));
			break;
		case 0:
		case 2:
			SetCursor(LoadCursor(NULL,IDC_SIZENWSE));
			break;
		case 1:
		case 3:
			SetCursor(LoadCursor(NULL,IDC_SIZENESW));
			break;
		}
		return 0;
	}


	void OnMouseMove(UINT nFlags, CPoint point)
	{
		if (mouse_click) {
			if (p_object != NULL) {
				// Move Object, Send Midi
				MoveObject(point);
				SendMidi(point);

				prev_point = point;

			} else {
				// Rect-Selection
				RectSelection(point);
			}

		} else {
			// Object under cursor to Pointer
			p_object = NULL;
			corner_num = -1;

			itLayer = vecLayer.begin();
			while (itLayer != vecLayer.end())
			{
				itObject = (*itLayer)->begin();
				while (itObject != (*itLayer)->end())
				{
					(*itObject)->SetPointer(point, p_object, corner_num);
					itObject++;
				}
				itLayer++;
			}
		}
	}


	void MoveObject(CPoint &point) {
		CRgn invalidate_rgn = CreateRectRgn(0,0,0,0);
		CRgn temp_rgn;				
		CRect client_rect;
		GetClientRect(client_rect);

		itLayer = vecLayer.begin();
		while (itLayer != vecLayer.end())
		{
			itObject = (*itLayer)->begin();
			while (itObject != (*itLayer)->end())
			{
				temp_rgn.CreateRectRgnIndirect((*itObject)->GetRect());
				(*itObject)->Move(point - prev_point, p_object, corner_num, client_rect);
				if ((*itObject)->GetMoved()) {
					invalidate_rgn.CombineRgn(temp_rgn, RGN_OR);
					temp_rgn.DeleteObject();
					temp_rgn.CreateRectRgnIndirect((*itObject)->GetRect());
					invalidate_rgn.CombineRgn(temp_rgn, RGN_OR);
				}
				temp_rgn.DeleteObject();
				itObject++;
			}
			itLayer++;
		}

		InvalidateRgn(invalidate_rgn, false);
		invalidate_rgn.DeleteObject();
	}


	void SendMidi(CPoint &point) {
		std::vector<Object *>::iterator itCH, itCC;
		float x, y;
		int value;
		CRect ch_rect, cc_rect;

		itCH = vecCH.begin();
		while (itCH != vecCH.end())
		{
			
			itCC = vecCC.begin();
			while (itCC != vecCC.end())
			{
				if (((*itCH)->GetMoved() || (*itCC)->GetMoved()) && 
					!((*itCH)->GetMoved() && (*itCC)->GetMoved())) {
					ch_rect.CopyRect((*itCH)->GetRect());
					cc_rect.CopyRect((*itCC)->GetRect());

					if (ch_rect.PtInRect(cc_rect.CenterPoint()) ||
						ch_rect.PtInRect(cc_rect.CenterPoint() - (point - prev_point)) ||
						ch_rect.PtInRect(cc_rect.CenterPoint() + (point - prev_point))) {
						x = (float)(ch_rect.CenterPoint().x - cc_rect.CenterPoint().x);
						y = (float)(ch_rect.CenterPoint().y - cc_rect.CenterPoint().y);
						if (x < 0) x = -x;
						if (y < 0) y = -y;
						if ((x / ch_rect.Width()) > 
							(y / ch_rect.Height())) {
							// Process X axis
							x = (ch_rect.Width()/2 - x) / (ch_rect.Width()/4) * 127;
							if (x > 127) x = 127;
							if (x < 0 ) x = 0;
							value = (int)x;
						} else {
							// Process Y axis
							y = (ch_rect.Height()/2 - y) / (ch_rect.Height()/4) * 127;
							if (y > 127) y = 127;
							if (y < 0) y = 0;
							value = (int)y;
						}
						midiOutShortMsg(midi_out, MIDIMSG((0xB0 + (*itCH)->GetNumber()), 
														  (0x00 + (*itCC)->GetNumber()), 
														  value));
					}
				}
				itCC++;
			}
			itCH++;
		}
	}


	void RectSelection(CPoint &point) {
		selection_rect.SetRect(prev_point, point);
		selection_rect.NormalizeRect();

		itLayer = vecLayer.begin();
		while (itLayer != vecLayer.end())
		{
			itObject = (*itLayer)->begin();
			while (itObject != (*itLayer)->end())
			{
				(*itObject)->InSelectionRect(selection_rect);
				itObject++;
			}
			itLayer++;
		}

		CRect client_rect;
		GetClientRect(client_rect);
		InvalidateRect(client_rect,false);
	}


	void DoPaint(CDCHandle dc)
	{
		// Background
        CRect client_rect;
        GetClientRect(client_rect);
        dc.FillSolidRect(client_rect,RGB(42,41,42));

		// Selection-Rect
		CBrush old_brush = dc.SelectBrush((HBRUSH)GetStockObject(DKGRAY_BRUSH));
		CPen pen = CreatePen(PS_SOLID, 1, RGB(80,80,80));
		CPen old_pen = dc.SelectPen(pen);
		dc.Rectangle(selection_rect);
		dc.SelectBrush(old_brush);
		dc.SelectPen(old_pen);
		pen.DeleteObject();

		// Objects
		dc.SetBkMode(TRANSPARENT);

		itLayer = vecLayer.begin();
		while (itLayer != vecLayer.end())
		{
			itObject = (*itLayer)->begin();
			while (itObject != (*itLayer)->end())
			{
				(*itObject)->Paint(dc);
				itObject++;
			}
			itLayer++;
		}
	}

};
